package pl.myapp.driver.sender.ws;

import java.net.MalformedURLException;
import java.net.URL;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.xml.ws.Service;
import javax.xml.ws.WebServiceException;
import javax.xml.ws.handler.Handler;
import javax.xml.ws.handler.HandlerResolver;
import javax.xml.ws.handler.PortInfo;
import javax.xml.ws.handler.soap.SOAPMessageContext;
import javax.xml.ws.soap.SOAPFaultException;

import org.apache.log4j.Logger;

@TransactionAttribute(value = TransactionAttributeType.SUPPORTS)
public class DasDriverWsSenderBean implements DasDriver, WebServiceSender {

    /**
     * Logger for class
     */
    private static final Logger logger = Logger.getLogger(DasDriverWsSenderBean.class);
    
    /**
     * DAS type
     */
    @Resource(name = "dasType")
    private String dasType;
    
    /**
     * WSDL location
     */
    @Resource(name = "wsdlLocation")
    private String wsdlLocation = null;
    
    /**
     * target namespace
     */
    @Resource(name = "targetNamespace")
    private String targetNamespace = null;
    
    /**
     * ServiceName
     */
    @Resource(name = "serviceName")
    private String serviceName = null;
    
    /**
     * ServicePort
     */
    @Resource(name = "servicePort")
    private String servicePort = null;
    
    /**
     * DAS
     */
    private DasWrapper das = null;
    
    @PostConstruct
    public void initializeWebServiceClient() throws DriverCommunicationLayerException {
        try {
            Service dasService;
            if (DasType.DAS_TYPE1.toString().equals(dasType)) {
                dasService = new DasService(new URL(this.wsdlLocation), this.targetNamespace, this.serviceName);
            } else if (DasType.DAS_TYPE2.toString().equals(dasType)) {
                dasService = new DasFaxService(new URL(this.wsdlLocation), this.targetNamespace, this.serviceName);
            } else {
                throw new DriverCommunicationLayerException(MessageFormat.format("Invalid DAS Web service configuration for wsdlLocation={0} and dasType={1}.", 
                                                                                 wsdlLocation, dasType));
            }
            
            logger.info("Driver SOAP client initialized");
            
        } catch (MalformedURLException mue) {
            throw new DriverCommunicationLayerException(MessageFormat.format("Invalid DAS Web service configuration for wsdlLocation={0} and dasType={1}. {2}", 
                                                                             wsdlLocation, dasType, mue.getMessage()), mue);
        } catch (WebServiceException wse) {
            throw new DriverCommunicationLayerException("Error while getting proxy port from service. " + wse.getMessage(), wse);
        }
    }
    
    public String getServiceName() {
        return this.serviceName;
    }

    public String getServicePort() {
        return this.servicePort;
    }

    public String getTargetNamespace() {
        return this.targetNamespace;
    }

    public String getWsdlLocation() {
        return this.wsdlLocation;
    }

}