package pl.myapp.driver.sender.ws;

import java.net.MalformedURLException;
import java.net.URL;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.xml.ws.WebServiceException;
import javax.xml.ws.handler.Handler;
import javax.xml.ws.handler.HandlerResolver;
import javax.xml.ws.handler.PortInfo;
import javax.xml.ws.handler.soap.SOAPMessageContext;

import org.apache.log4j.Logger;

@TransactionAttribute(value = TransactionAttributeType.SUPPORTS)
public class FesDriverWsSenderBean implements FesDriver, WebServiceSender {

    /**
     * Logger for class
     */
    private static final Logger logger = Logger.getLogger(FesDriverWsSenderBean.class);

    /**
     * WSDL location
     */
    @Resource(name = "wsdlLocation")
    private String wsdlLocation;

    /**
     * target namespace
     */
    @Resource(name = "targetNamespace")
    private String targetNamespace;

    /**
     * ServiceName
     */
    @Resource(name = "serviceName")
    private String serviceName;

    /**
     * ServicePort
     */
    @Resource(name = "servicePort")
    private String servicePort;

    /**
     * actual driver
     */
    private FesDriver driver;

    public String getServiceName() {
        return this.serviceName;
    }

    public String getServicePort() {
        return this.servicePort;
    }

    public String getTargetNamespace() {
        return this.targetNamespace;
    }

    public String getWsdlLocation() {
        return this.wsdlLocation;
    }

    @PostConstruct
    public void initializeWebServiceClient() throws DriverCommunicationLayerException {
        try {
            FesService feService = new FesService(new URL(this.wsdlLocation), this.targetNamespace, this.serviceName);

            logger.info("Driver SOAP client initialized");
        } catch (MalformedURLException mue) {
            throw new DriverCommunicationLayerException(MessageFormat.format(
                    "Invalid FE Web service configuration for wsdlLocation={0}. {1}", wsdlLocation, mue.getMessage()),
                    mue);
        } catch (WebServiceException wse) {
            throw new DriverCommunicationLayerException("Error while getting proxy port from service. "
                    + wse.getMessage(), wse);
        }
    }

}
